<?php
declare(strict_types = 1);

namespace Ig\IgMenu\DataProcessing;

use Ig\IgMenu\Source\DataProcessorInterface;
use Ig\IgMenu\Domain\Model\MenuItem;
use Ig\IgMenu\Domain\Repository\PageRepository;
use Ig\IgMenu\Utility\SettingsUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\Restriction\FrontendRestrictionContainer;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use TYPO3Fluid\Fluid\Core\Rendering\RenderingContextInterface;

use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;

class PagesProcessor  implements DataProcessorInterface,SingletonInterface
{
    /**
     * PageRepository
     *
     * @var \Ig\IgMenu\Domain\Repository\PageRepository
     */
    public $pageRepository = null;


    protected $doktypesInMenu = [1, 4, 3, 7];
    function __construct() {
        $this->conn = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('pages');
        $this->queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');        
    }

    /**
     * @param \Ig\IgMenu\Domain\Repository\PageRepository $pageRepository
     */
    public function injectPageRepository(PageRepository $pageRepository)
    {
        $this->pageRepository = $pageRepository;
    }
    
    /**
     * Process first call $menuItem is a page
     *
     * @param MenuItem $menuItem - the current menuItem (data is a page)
     */
    public function process( MenuItem $menuItem )
    {
        return $this->getSubMenuItems($menuItem);
    }
    
    /**
     * Process subitems - $menuItem is a category
     *
     * @param MenuItem $menuItem - the current menuItem
     */
    public function getChildren( MenuItem $menuItem )
    {
        return $this->getSubMenuItems($menuItem);
    }

    public function addDoktypesInMenu(array $addDoktypesInMenu)
    {
        $this->doktypesInMenu = array_merge($this->doktypesInMenu, $addDoktypesInMenu);
    
    }
    public function removeDoktypesInMenu(array $removeDoktypesInMenu)
    {
        $this->doktypesInMenu = array_diff($this->doktypesInMenu, $removeDoktypesInMenu);
    }
    public function getDoktypesInMenu():array
    {
        return  $this->doktypesInMenu;
    }
    public function setDoktypesInMenu(array $doktypesInMenu)
    {
        $this->doktypesInMenu = $doktypesInMenu;
    }
    
    /**
     * convert to menuItems
     *
     */
    /*
      public function getMenuItemsByPageUid($pageUid) {
      $additionalQuery = $this->getAdditionalQuery();
      $menu = $this->pageRepository->getIgMenu($pageUid, 'uid', 'sorting', $additionalQuery);
      return $this->getMenuItems($menu, null);
      }
    */
    public function getSubMenuItems($menuItem) {
        $additionalQuery = $this->getAdditionalQuery();
        if ($menuItem->getDoktype() == 7 && $menuItem->getMountPid()) {
            $menu = $this->pageRepository->getIgMenu($menuItem->getMountPid(), 'uid', 'sorting', $additionalQuery);
            //$mpInfo = $this->pageRepository->getMountPointInfo($menuItem->getUid());
            //$menuItem->set_MP_PARAM($mpInfo['MPvar']);
            $menuItem->set_MP_PARAM($menuItem->getMountPid() . '-' . $menuItem->getUid());
            //_mount_pid_ol
            if ($menuItem->getMountPidOl() == 1) {
                $page = $this->pageRepository->getPage($menuItem->getMountPid());
                $menuItem->mergeIgMenuAttributes($page);//@todo
            }
        } else {
            $menu = $this->pageRepository->getIgMenu($menuItem->getUid(), 'uid', 'sorting', $additionalQuery);
        }
        return $this->getMenuItems($menu, $menuItem->get_MP_PARAM());
    }

        /**
     * convert to menuItems
     *
     */
    public function getMenuItems($menu, $mpParam) {
        $menuItems = [];
        $totalWidth = 0;
        $anzahlEintraege = 0;
        foreach ($menu as $menuEntry) {
            $menuItem = GeneralUtility::makeInstance(MenuItem::class);
            $menuItem->init($this->pageRepository->getPage($menuEntry['uid']));
            $totalWidth += $menuItem->getIgMenuColWidth();
            $anzahlEintraege++;
            if ($mpParam) {
                $menuItem->set_MP_PARAM($mpParam);
            }

            $menuItems[] = $menuItem;
        }

        return $menuItems;
    }

    public function getAdditionalQuery()
    {
        $expressionBuilder = $this->conn->getExpressionBuilder();
        $frontendRestriction = GeneralUtility::makeInstance(FrontendRestrictionContainer::class);

        $this->queryBuilder->select('*')->from('pages');
        //$this->queryBuilder->setRestrictions($frontendRestriction);
        $this->queryBuilder->where($frontendRestriction->buildExpression(['pages' => 'pages'], $expressionBuilder));
        $menuConf = GeneralUtility::makeInstance(SettingsUtility::class);

        if (!$menuConf->getArgument('includeNotInMenu')) {
            $this->queryBuilder->andWhere($expressionBuilder->eq('nav_hide', '0'));
        }
        $pagesToExclude = $menuConf->getArgument('pagesToExclude');
        if ($pagesToExclude) {
            $pagesToExcludeArray = array_map('trim', explode(',', (string) $pagesToExclude));
            $this->queryBuilder->andWhere($expressionBuilder->notIn('uid', $pagesToExcludeArray));
        }

        $this->queryBuilder->andWhere($expressionBuilder->in('doktype', $this->doktypesInMenu))
            ->andWhere($expressionBuilder->eq('nav_hide', '0'));
        return $this->queryBuilder->getQueryPart('where')->__toString();
    }
    /**
     * convert to menuItems
     *
     */
    public function convertCategories($pageUid, $categories) {
        $menuitems = [];
        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
        foreach($categories as $category) {
            $menuitem =  GeneralUtility::makeInstance(MenuItem::class);
            $menuitem->setTitle($category->getName());
            $menuitem->setUid($pageUid);
            $menuitem->setData($category);
            $menuitem->setActiveKey('category-' .$category->getUid());
            $uriBuilder
                ->reset()
                ->setTargetPageUid($pageUid);
            //->setArguments(['category' => $category->getUid()]);
            $menuitem->setLink($uriBuilder->uriFor(
                'list', // actionName
                ['category' => $category->getUid()], //controllerArguments
                'Product', // controllerName
                'igshop2', // extensionName
                'Feshop' // pluginName
            ));
                                
            $menuitems[] = $menuitem;
        }
        return $menuitems;
    }

    public function getStoragePid()
    {
        $configurationManager = GeneralUtility::makeInstance(ConfigurationManager::class);
        return $configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT)['plugin.']['tx_igshop2.']['settings.']['storagePid'];
    }

    /**
     * Get Active Entries (Parameters with values)
     *
     * @param int $pageUid
     * @param array $queryParams
     */
    public function getActiveKeys(int $pageUid, array $queryParams)
    {
        $activeKeys = [];
        if(isset($queryParams['tx_igshop2_feshop']['category'])) {
            $category = (int) $queryParams['tx_igshop2_feshop']['category'];
            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('tx_igshop2_domain_model_category');
            $limit = 10;
            while( $category>0 && $limit>0) {
                $limit--;
                $activeKeys[] = 'category-' . $category;
                $result = $queryBuilder
                        ->select('parent')
                        ->from('tx_igshop2_domain_model_category')
                        ->where(
                            $queryBuilder->expr()->eq(
                                'uid',
                                $queryBuilder->createNamedParameter($category, \PDO::PARAM_INT)
                            )
                        )
                        ->execute()->fetch();
                $category = (int) $result['parent'];
            }
        }
        return $activeKeys;
    }
        
}
